import django.db.models.deletion
import obapi.modelfields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Author",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(help_text="Name.", max_length=100, unique=True),
                ),
                (
                    "slug",
                    obapi.modelfields.SimpleSlugField(
                        editable=False, max_length=150, unique=True
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ContentItem",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "create_timestamp",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="creation date"
                    ),
                ),
                (
                    "update_timestamp",
                    models.DateTimeField(auto_now=True, verbose_name="update date"),
                ),
                (
                    "download_timestamp",
                    models.DateTimeField(
                        editable=False, null=True, verbose_name="download date"
                    ),
                ),
                (
                    "title",
                    models.CharField(help_text="Title of content.", max_length=100),
                ),
                (
                    "description_html",
                    models.TextField(
                        blank=True,
                        help_text="HTML description of content.",
                        max_length=5000,
                    ),
                ),
                (
                    "publish_date",
                    models.DateTimeField(help_text="Date of publication."),
                ),
                (
                    "edit_date",
                    models.DateTimeField(
                        blank=True, help_text="Date of last edit.", null=True
                    ),
                ),
                (
                    "authors",
                    models.ManyToManyField(
                        help_text="Authors.", related_name="content", to="obapi.author"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ExternalLink",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("url", models.URLField(help_text="Link URL.", unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="Idea",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(help_text="Name.", max_length=100, unique=True),
                ),
                (
                    "slug",
                    obapi.modelfields.SimpleSlugField(
                        editable=False, max_length=150, unique=True
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        blank=True, help_text="Brief description.", max_length=100
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Sequence",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(help_text="Sequence title.", max_length=100),
                ),
                (
                    "slug",
                    obapi.modelfields.SimpleSlugField(editable=False, max_length=150),
                ),
                (
                    "abstract",
                    models.TextField(
                        blank=True,
                        help_text="Description of sequence.",
                        max_length=5000,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(help_text="Name.", max_length=100, unique=True),
                ),
                (
                    "slug",
                    obapi.modelfields.SimpleSlugField(
                        editable=False, max_length=150, unique=True
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Topic",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(help_text="Name.", max_length=100, unique=True),
                ),
                (
                    "slug",
                    obapi.modelfields.SimpleSlugField(
                        editable=False, max_length=150, unique=True
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        blank=True, help_text="Brief description.", max_length=100
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AudioContentItem",
            fields=[
                (
                    "contentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.contentitem",
                    ),
                ),
                (
                    "duration",
                    models.DurationField(
                        blank=True, help_text="Duration of episode.", null=True
                    ),
                ),
                (
                    "listen_count",
                    models.PositiveIntegerField(
                        blank=True, help_text="Number of episode listens.", null=True
                    ),
                ),
            ],
            options={
                "verbose_name": "episode",
            },
            bases=("obapi.contentitem",),
        ),
        migrations.CreateModel(
            name="TextContentItem",
            fields=[
                (
                    "contentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.contentitem",
                    ),
                ),
                (
                    "word_count",
                    models.PositiveIntegerField(
                        blank=True, help_text="Word count.", null=True
                    ),
                ),
                (
                    "text_html",
                    models.TextField(
                        blank=True,
                        help_text="Content text HTML.",
                        verbose_name="content text HTML",
                    ),
                ),
                (
                    "text_plain",
                    models.TextField(
                        blank=True,
                        help_text="Content plaintext.",
                        verbose_name="content plaintext",
                    ),
                ),
            ],
            bases=("obapi.contentitem",),
        ),
        migrations.CreateModel(
            name="VideoContentItem",
            fields=[
                (
                    "contentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.contentitem",
                    ),
                ),
                (
                    "duration",
                    models.DurationField(
                        blank=True, help_text="Duration of video.", null=True
                    ),
                ),
                (
                    "view_count",
                    models.PositiveIntegerField(
                        blank=True, help_text="Number of video views.", null=True
                    ),
                ),
            ],
            options={
                "verbose_name": "video",
            },
            bases=("obapi.contentitem",),
        ),
        migrations.CreateModel(
            name="TopicAlias",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    obapi.modelfields.SimpleSlugField(
                        help_text="Alias text.", max_length=150, unique=True
                    ),
                ),
                (
                    "protected",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="True for the alias which equals the owner name.",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Topic the alias refers to.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="aliases",
                        related_query_name="alias",
                        to="obapi.topic",
                        verbose_name="topic",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "aliases",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TagAlias",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    obapi.modelfields.SimpleSlugField(
                        help_text="Alias text.", max_length=150, unique=True
                    ),
                ),
                (
                    "protected",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="True for the alias which equals the owner name.",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Tag the alias refers to.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="aliases",
                        related_query_name="alias",
                        to="obapi.tag",
                        verbose_name="tag",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "aliases",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="SequenceMember",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "order",
                    models.PositiveIntegerField(
                        db_index=True, editable=False, verbose_name="order"
                    ),
                ),
                (
                    "content_item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sequence_members",
                        related_query_name="sequence_members",
                        to="obapi.contentitem",
                    ),
                ),
                (
                    "sequence",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="members",
                        related_query_name="members",
                        to="obapi.sequence",
                    ),
                ),
            ],
            options={
                "ordering": ("sequence", "order"),
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="sequence",
            name="items",
            field=models.ManyToManyField(
                through="obapi.SequenceMember", to="obapi.contentitem"
            ),
        ),
        migrations.CreateModel(
            name="IdeaAlias",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    obapi.modelfields.SimpleSlugField(
                        help_text="Alias text.", max_length=150, unique=True
                    ),
                ),
                (
                    "protected",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="True for the alias which equals the owner name.",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Idea the alias refers to.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="aliases",
                        related_query_name="alias",
                        to="obapi.idea",
                        verbose_name="idea",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "aliases",
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="contentitem",
            name="external_links",
            field=models.ManyToManyField(
                blank=True,
                help_text="Links to external webpages.",
                related_name="content",
                related_query_name="content",
                to="obapi.externallink",
            ),
        ),
        migrations.AddField(
            model_name="contentitem",
            name="ideas",
            field=models.ManyToManyField(
                blank=True,
                help_text="Ideas discussed or referenced by the content.",
                related_name="content",
                related_query_name="content",
                to="obapi.idea",
            ),
        ),
        migrations.AddField(
            model_name="contentitem",
            name="internal_links",
            field=models.ManyToManyField(
                blank=True,
                help_text="Links to other content items.",
                related_name="internal_pingbacks",
                related_query_name="internal_pingbacks",
                to="obapi.contentitem",
            ),
        ),
        migrations.AddField(
            model_name="contentitem",
            name="tags",
            field=models.ManyToManyField(
                blank=True,
                help_text="Miscellaneous tags.",
                related_name="content",
                related_query_name="content",
                to="obapi.tag",
            ),
        ),
        migrations.AddField(
            model_name="contentitem",
            name="topics",
            field=models.ManyToManyField(
                blank=True,
                help_text="Topics discussed or references by the content.",
                related_name="content",
                related_query_name="content",
                to="obapi.topic",
            ),
        ),
        migrations.CreateModel(
            name="AuthorAlias",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    obapi.modelfields.SimpleSlugField(
                        help_text="Alias text.", max_length=150, unique=True
                    ),
                ),
                (
                    "protected",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="True for the alias which equals the owner name.",
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Author the alias refers to.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="aliases",
                        related_query_name="alias",
                        to="obapi.author",
                        verbose_name="author",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "aliases",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="OBContentItem",
            fields=[
                (
                    "textcontentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.textcontentitem",
                    ),
                ),
                (
                    "item_id",
                    models.CharField(
                        help_text='Post string identifier. E.g. "2006/11/introduction"',
                        max_length=100,
                        unique=True,
                        verbose_name="string ID",
                    ),
                ),
                (
                    "ob_post_number",
                    models.PositiveIntegerField(
                        help_text="Post number identifier.",
                        unique=True,
                        verbose_name="number ID",
                    ),
                ),
                (
                    "disqus_id",
                    models.CharField(
                        blank=True,
                        help_text="Post Disqus API string identifier.",
                        max_length=200,
                    ),
                ),
                (
                    "ob_likes",
                    models.PositiveSmallIntegerField(
                        blank=True,
                        help_text="Number of likes.",
                        null=True,
                        verbose_name="likes",
                    ),
                ),
                (
                    "ob_comments",
                    models.PositiveSmallIntegerField(
                        blank=True,
                        help_text="Number of comments.",
                        null=True,
                        verbose_name="comments",
                    ),
                ),
            ],
            options={
                "verbose_name": "overcomingbias post",
            },
            bases=("obapi.textcontentitem",),
        ),
        migrations.CreateModel(
            name="SpotifyContentItem",
            fields=[
                (
                    "audiocontentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.audiocontentitem",
                    ),
                ),
                (
                    "item_id",
                    models.CharField(
                        help_text="Episode ID.",
                        max_length=30,
                        unique=True,
                        verbose_name="episode ID",
                    ),
                ),
                (
                    "sp_show_id",
                    models.CharField(
                        help_text="Show ID.", max_length=30, verbose_name="show ID"
                    ),
                ),
                (
                    "sp_show_title",
                    models.CharField(
                        help_text="Show title.",
                        max_length=200,
                        verbose_name="show title",
                    ),
                ),
                (
                    "sp_description",
                    models.TextField(
                        blank=True,
                        help_text="Episode description.",
                        max_length=5000,
                        verbose_name="description",
                    ),
                ),
            ],
            options={
                "verbose_name": "spotify episode",
            },
            bases=("obapi.audiocontentitem",),
        ),
        migrations.CreateModel(
            name="YoutubeContentItem",
            fields=[
                (
                    "videocontentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.videocontentitem",
                    ),
                ),
                (
                    "item_id",
                    models.CharField(
                        help_text="Video ID.",
                        max_length=30,
                        unique=True,
                        verbose_name="video ID",
                    ),
                ),
                (
                    "yt_channel_id",
                    models.CharField(
                        help_text="Channel ID.",
                        max_length=30,
                        verbose_name="channel ID",
                    ),
                ),
                (
                    "yt_channel_title",
                    models.CharField(
                        help_text="Channel title.",
                        max_length=30,
                        verbose_name="channel title",
                    ),
                ),
                (
                    "yt_likes",
                    models.PositiveIntegerField(
                        blank=True,
                        help_text="Number of likes.",
                        null=True,
                        verbose_name="likes",
                    ),
                ),
                (
                    "yt_description",
                    models.TextField(
                        blank=True,
                        help_text="Video description.",
                        max_length=5000,
                        verbose_name="description",
                    ),
                ),
            ],
            options={
                "verbose_name": "youtube video",
            },
            bases=("obapi.videocontentitem",),
        ),
        migrations.AddConstraint(
            model_name="sequence",
            constraint=models.UniqueConstraint(
                fields=("slug",), name="unique_sequence_slug"
            ),
        ),
    ]



from django import forms

from obapi.models import OBContentItem, SpotifyContentItem, YoutubeContentItem
from obapi.models.content import EssayContentItem

__all__ = [
    "AddContentItemForm",
    "AddYoutubeContentItemForm",
    "AddSpotifyContentItemForm",
    "AddOBContentItemForm",
]


class AddContentItemForm(forms.ModelForm):
    class Meta:
        fields = ("item_id",)

    def save(self):
        item_id = self.cleaned_data.pop("item_id")
        return self.Meta.model.objects.create_item(item_id)


class AddYoutubeContentItemForm(AddContentItemForm):
    class Meta(AddContentItemForm.Meta):
        model = YoutubeContentItem


class AddSpotifyContentItemForm(AddContentItemForm):
    class Meta(AddContentItemForm.Meta):
        model = SpotifyContentItem


class AddOBContentItemForm(AddContentItemForm):
    class Meta(AddContentItemForm.Meta):
        model = OBContentItem


class AddEssayContentItemForm(AddContentItemForm):
    class Meta(AddContentItemForm.Meta):
        model = EssayContentItem



import datetime

import pytest
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from obapi.models import (
    SEQUENCE_SLUG_MAX_LENGTH,
    EssayContentItem,
    Sequence,
    SpotifyContentItem,
    YoutubeContentItem,
)
from obapi.utils import to_slug


@pytest.mark.django_db
class TestCreateSequence:
    def test_can_create_empty_sequence(self):
        # Arrange
        seq_title = "Empty Sequence"
        # Act
        Sequence.objects.create(title=seq_title, abstract="Description")
        # Assert
        seq = Sequence.objects.get(title=seq_title)
        assert seq.slug == to_slug(seq_title, max_length=SEQUENCE_SLUG_MAX_LENGTH)
        assert not seq.items.all().exists()

        # Act - modify title
        new_title = "New Title"
        seq.title = new_title
        seq.save()
        assert seq.slug == to_slug(new_title, max_length=SEQUENCE_SLUG_MAX_LENGTH)

    def test_can_create_normal_sequence(self):
        # Arrange
        now = datetime.datetime.now()
        video = YoutubeContentItem.objects.create(
            title="YouTube Item", item_id="4yZKGbq1YmA", publish_date=now
        )
        audio = SpotifyContentItem.objects.create(
            title="Spotify Item", item_id="6z8lvJia3OzcIVCStbmEtU", publish_date=now
        )
        text = EssayContentItem.objects.create(
            title="Essay Item", item_id="example", publish_date=now
        )
        seq_title = "Example Sequence"

        # Act - create basic sequence
        seq = Sequence.objects.create(title=seq_title)
        seq.items.add(video, audio, text)

        # Assert
        seq = Sequence.objects.get(title=seq_title)
        assert seq.title == seq_title
        assert seq.items.count() == 3

        # Act - delete one item
        video.delete()
        del video

        # Assert
        seq = Sequence.objects.get(title=seq_title)
        assert seq.title == seq_title
        assert seq.items.count() == 2


@pytest.mark.django_db
class TestSlugUniquenessIsEnforced:
    def test_duplicate_slug_does_not_save(self):
        # Act & Assert - sequence with owner
        Sequence.objects.create(title="Example Sequence")
        with pytest.raises(IntegrityError):
            Sequence.objects.create(title="example-Sequence")

    def test_duplicate_slug_does_not_validate(self):
        # Act & Assert
        Sequence.objects.create(title="Example Sequence")
        seq2 = Sequence(title="example-Sequence")
        with pytest.raises(ValidationError):
            seq2.full_clean()



# Generated by Django 4.0.3 on 2022-07-11 11:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0005_sequence_create_timestamp_sequence_update_timestamp"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="sequence",
            options={"ordering": ("-update_timestamp",)},
        ),
    ]



from obapi.models.classifiers import (
    CLASSIFIER_SLUG_MAX_LENGTH,
    Author,
    AuthorAlias,
    ExternalLink,
    Idea,
    IdeaAlias,
    Tag,
    TagAlias,
    Topic,
    TopicAlias,
)
from obapi.models.content import (
    ContentItem,
    EssayContentItem,
    OBContentItem,
    SpotifyContentItem,
    YoutubeContentItem,
)
from obapi.models.sequence import (
    SEQUENCE_SLUG_MAX_LENGTH,
    BaseSequence,
    BaseSequenceMember,
    Sequence,
    SequenceMember,
)

__all__ = [
    "CLASSIFIER_SLUG_MAX_LENGTH",
    "Author",
    "Idea",
    "Topic",
    "Tag",
    "TagAlias",
    "AuthorAlias",
    "IdeaAlias",
    "TopicAlias",
    "ExternalLink",
    "ContentItem",
    "EssayContentItem",
    "YoutubeContentItem",
    "SpotifyContentItem",
    "OBContentItem",
    "SEQUENCE_SLUG_MAX_LENGTH",
    "BaseSequence",
    "BaseSequenceMember",
    "Sequence",
    "SequenceMember",
]



import pytest
from django.conf import settings

require_youtube_api_key = pytest.mark.skipif(
    settings.YOUTUBE_API_KEY is None,
    reason="Requires a valid YouTube API key to be set.",
)

require_spotify_api_auth = pytest.mark.skipif(
    settings.SPOTIFY_CLIENT_ID is None and settings.SPOTIFY_CLIENT_SECRET is None,
    reason="Requires a valid Spotify client ID and secret to be set.",
)



# Generated by Django 4.0.3 on 2022-07-07 14:20

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0003_alter_contentitem_title_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="EssayContentItem",
            fields=[
                (
                    "textcontentitem_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="obapi.textcontentitem",
                    ),
                ),
                (
                    "item_id",
                    models.CharField(
                        help_text='Essay string identifier. E.g. "Varytax"',
                        max_length=300,
                        unique=True,
                        verbose_name="string ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "essay",
            },
            bases=("obapi.textcontentitem",),
        ),
    ]



#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import os
import sys

import django
from django.core.management import call_command, execute_from_command_line


def main():
    """Run administrative tasks."""
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.test_settings")
    django.setup()
    call_command("migrate")
    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    main()



import pytest
from obapi.utils import parse_duration, plaintext_to_html


class TestParseDuration:
    def test_gives_correct_result_for_valid_durations(self):
        assert parse_duration("PT10S").total_seconds() == pytest.approx(10)
        assert parse_duration("P3DT3H5M2S").total_seconds() == pytest.approx(270302)

    def test_raises_error_for_invalid_durations(self):
        with pytest.raises(ValueError):
            parse_duration("arbitrary string")

        with pytest.raises(ValueError):
            parse_duration("PT3.827M")


class TestPlaintextToHTML:
    def test_removes_malicious_text(self):
        # Arrange
        malicious_text = "A text with an evil <script></script>"
        # Act
        html_text = plaintext_to_html(malicious_text)
        # Assert
        assert "<script>" not in html_text

    def test_linkifies_urls(self):
        text_with_url = "A www.example.com URL"
        html_text = plaintext_to_html(text_with_url)
        assert "</a>" in html_text

    def test_wraps_with_pre_tag(self):
        text = "Some sample text to be wrapped with pre tags"
        html_text = plaintext_to_html(text)
        assert html_text.startswith("<pre>")
        assert html_text.endswith("</pre>")



"""Download and tidy API data.

These functions return ContentItem data in a standard format understood by the
ContentItemManager.
"""
import datetime
from functools import wraps

from obapi.download import (
    download_essays,
    download_ob_edit_dates,
    download_ob_post_objects,
    download_spotify_episodes_json,
    download_youtube_videos_json,
)
from obapi.tidy import (
    tidy_essays,
    tidy_ob_post_objects,
    tidy_spotify_episodes_json,
    tidy_youtube_videos_json,
)


def download_timestamp(decorated_function):
    """Attach a timestamp to result of a function.

    Attaches a `download_timestamp` attribute to the result of the function, which
    specifies the time when the function was called.

    If the output of the function is not a list of dictionaries (e.g. if it is None),
    it does nothing.
    """

    @wraps(decorated_function)
    def decorator(*args, **kwargs):
        download_timestamp = datetime.datetime.now(datetime.timezone.utc)
        result = decorated_function(*args, **kwargs)
        if isinstance(result, list):
            for item in result:
                if isinstance(item, dict):
                    item["download_timestamp"] = download_timestamp
        return result

    return decorator


@download_timestamp
def assemble_youtube_content_items(video_ids):
    raw_json = download_youtube_videos_json(video_ids)
    tidy_dict = tidy_youtube_videos_json(video_ids, raw_json)
    return tidy_dict


@download_timestamp
def assemble_spotify_content_items(episode_ids):
    raw_json = download_spotify_episodes_json(episode_ids)
    tidy_dict = tidy_spotify_episodes_json(episode_ids, raw_json)
    return tidy_dict


@download_timestamp
def assemble_ob_content_items(post_names):
    post_dict = download_ob_post_objects(post_names)
    tidy_dict = tidy_ob_post_objects(post_names, post_dict)
    return tidy_dict


@download_timestamp
def assemble_essay_content_items(essay_ids):
    essay_dict = download_essays(essay_ids)
    tidy_dict = tidy_essays(essay_ids, essay_dict)
    return tidy_dict


def assemble_ob_edit_dates():
    return download_ob_edit_dates()



from django.core.exceptions import ValidationError
from django.db import models, transaction
from django.urls import reverse
from obapi import utils
from obapi.modelfields import SimpleSlugField

CLASSIFIER_SLUG_MAX_LENGTH = 150


class AliasedModelQuerySet(models.QuerySet):
    def create_with_aliases(self, aliases=None, **kwargs):
        """Save an object with some aliases.

        Assumes that the given aliases have been cleaned.
        """
        aliases = set(aliases)
        with transaction.atomic():
            new_object = self.create(**kwargs)
            aliases.discard(new_object.slug)
            for alias in aliases:
                new_object.aliases.create(text=alias)
        return new_object

    def merge_objects(self):
        """Merge a QuerySet of objects."""
        new_fields = {}

        # Make list of aliases
        alias_model = self.model.aliases.field.model
        aliases_queryset = alias_model.objects.filter(owner__in=self)
        aliases = set(aliases_queryset.values_list("text", flat=True))

        # Use name of first object in QuerySet
        new_fields["name"] = self.first().name

        # Create new description (if description field exists)
        try:
            max_length = self.model.description.field.max_length
        except AttributeError:
            pass
        else:
            description_list = self.values_list("description", flat=True)
            description = "/".join([desc for desc in description_list if desc != ""])
            new_fields["description"] = description[0:max_length]

        with transaction.atomic():
            # Collect related content
            contentitem_field = self.model.content.field
            items = list(
                contentitem_field.model.objects.filter(
                    **{f"{contentitem_field.name}__in": self}
                ).distinct()
            )

            # Delete old object, create new
            self.delete()
            new_object = self.create_with_aliases(aliases=aliases, **new_fields)

            # Add related content
            new_object.content.add(*items)

        return new_object


class AliasedModel(models.Model):
    """Base class for models with aliases."""

    objects = AliasedModelQuerySet().as_manager()

    name = models.CharField(max_length=100, unique=True, help_text="Name.")
    slug = SimpleSlugField(
        max_length=CLASSIFIER_SLUG_MAX_LENGTH, unique=True, editable=False
    )
    description = models.CharField(
        max_length=100, help_text="Brief description.", blank=True
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    def clean(self):
        # Set slug from title
        self.slug = utils.to_slug(self.name, max_length=CLASSIFIER_SLUG_MAX_LENGTH)
        super().clean()

    def save(self, *args, **kwargs):
        """Save an AliasedModel instance.

        Raises
        ------
        IntegrityError
            If the instance `name` is already an alias of another instance.
        """
        # (1) Clean and set slug
        self.clean()
        with transaction.atomic():
            # (2) Save model instance
            super().save(*args, **kwargs)
            # (3) Set and protect `slug` alias
            self.aliases.all().update(protected=False)
            self.aliases.update_or_create(
                text=self.slug,
                defaults={"protected": True},
            )

    def get_absolute_url(self):
        model_name = self._meta.verbose_name
        return reverse(
            "explore_detail",
            kwargs={"model_name": model_name, "instance_name": self.slug},
        )

    def validate_unique(self, exclude=None):
        super().validate_unique(exclude=exclude)
        if exclude is not None and "text" in exclude:
            return
        # Raise error if there is a matching alias, and the match has a different pk
        alias_model = type(self).aliases.field.model
        try:
            match = alias_model.objects.get(
                text=utils.to_slug(self.name, max_length=CLASSIFIER_SLUG_MAX_LENGTH)
            )
        except alias_model.DoesNotExist:
            return
        else:
            if match.owner != self:
                self.refresh_from_db(fields=["name"])
                raise ValidationError(
                    {"name": "The alias of this name is already taken."}, code="invalid"
                )

    def convert_object(self, model):
        """Convert object to another type."""
        with transaction.atomic():
            # Initialise fields
            new_fields = {
                "name": self.name,
                "aliases": self.aliases.values_list("text", flat=True),
            }
            if model.description:
                if (old_description := self.description) is None:
                    new_fields["description"] = ""
                else:
                    new_fields["description"] = old_description

            # Create new object
            new_object = model.objects.create_with_aliases(**new_fields)

            # Update related content
            items = self.content.all()
            new_object.content.add(*items)

            # Delete old object
            self.delete()

        return new_object


class Author(AliasedModel):
    description = None


class Idea(AliasedModel):
    pass


class Topic(AliasedModel):
    pass


class Tag(AliasedModel):
    description = None


class Alias(models.Model):
    """Base class for models which represent aliases of other models.

    `owner` is the Foreign Key to the aliased model. The aliased model should have a
    `name` attribute, or the subclass should override the __str__ method.
    """

    text = SimpleSlugField(
        max_length=CLASSIFIER_SLUG_MAX_LENGTH, help_text="Alias text.", unique=True
    )
    protected = models.BooleanField(
        default=False,
        editable=False,
        help_text="True for the alias which equals the owner name.",
    )
    owner = None

    class Meta:
        abstract = True
        verbose_name_plural = "aliases"

    def __str__(self):
        return self.text

    def clean(self):
        self.text = utils.to_slug(self.text, max_length=CLASSIFIER_SLUG_MAX_LENGTH)
        super().clean()

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)


class AuthorAlias(Alias):
    owner = models.ForeignKey(
        Author,
        verbose_name="author",
        on_delete=models.CASCADE,
        related_name="aliases",
        related_query_name="alias",
        help_text="Author the alias refers to.",
    )


class IdeaAlias(Alias):
    owner = models.ForeignKey(
        Idea,
        verbose_name="idea",
        on_delete=models.CASCADE,
        related_name="aliases",
        related_query_name="alias",
        help_text="Idea the alias refers to.",
    )


class TopicAlias(Alias):
    owner = models.ForeignKey(
        Topic,
        verbose_name="topic",
        on_delete=models.CASCADE,
        related_name="aliases",
        related_query_name="alias",
        help_text="Topic the alias refers to.",
    )


class TagAlias(Alias):
    owner = models.ForeignKey(
        Tag,
        verbose_name="tag",
        on_delete=models.CASCADE,
        related_name="aliases",
        related_query_name="alias",
        help_text="Tag the alias refers to.",
    )


class ExternalLink(models.Model):
    """Model representing link to "external" content.

    Here "external" means "outside the database".
    """

    url = models.URLField(max_length=2048, help_text="Link URL.")

    def __str__(self):
        return self.url



from django import forms

import obapi.export
import obapi.validators


class SimpleSlugField(forms.CharField):
    default_validators = [obapi.validators.validate_simple_slug]


class PandocWriterField(forms.ChoiceField):
    """A form field for selecting a Sequence export format."""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.writers = {
            Writer.pandoc_name: Writer
            for Writer in obapi.export.get_supported_writers()
        }

        self.choices = [
            (writer_name, writer_name) for writer_name in self.writers.keys()
        ]

    def clean(self, value):
        value = super().clean(value)
        try:
            return self.writers[value]
        except KeyError:
            raise forms.ValidationError(
                self.error_messages["invalid_choice"],
                code="invalid_choice",
                params={"value": value},
            )



# Generated by Django 4.0.3 on 2022-07-10 23:17

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0004_essaycontentitem"),
    ]

    operations = [
        migrations.AddField(
            model_name="sequence",
            name="create_timestamp",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                help_text="When the sequence was created.",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="sequence",
            name="update_timestamp",
            field=models.DateTimeField(
                auto_now=True,
                help_text="When the sequence was last updated.",
                verbose_name="update date",
            ),
        ),
    ]



import os
from dataclasses import dataclass
from typing import Optional, Union

import pandadoc
from django.template.loader import render_to_string


def get_supported_writers():
    # Available pandoc output formats
    available_writers = pandadoc.call_pandoc(["--list-output-formats"]).splitlines()
    supported_writers = [
        Writer
        for Writer in CONFIGURED_WRITERS
        if Writer.pandoc_name in available_writers
    ]

    return supported_writers


def get_export_template(item):
    pandoc_template_format = ".html"
    opts = item._meta
    return f"{opts.app_label}/export/{opts.model_name}{pandoc_template_format}"


def render_item(item):
    """Render a ContentItem using a template."""
    export_template = get_export_template(item)

    return render_to_string(export_template, {"item": item})


def render_sequence(items):
    return "\n".join([render_item(item) for item in items])


def export_sequence(sequence, writer):
    # Metadata
    # metadata_template = f"{sequence._meta.app_label}/export/metadata.md"
    # metadata = render_to_string(metadata_template, {"sequence": sequence})

    content_item_pks = [
        member.content_item_id for member in sequence.members.order_by("order")
    ]
    item_dict = sequence.items.select_subclasses().in_bulk()
    items = [item_dict[pk] for pk in content_item_pks]

    sequence_html = render_sequence(items)

    return writer.convert(input_format="html", input_text=sequence_html)


@dataclass
class PandocWriter:
    """Base class for representing pandoc config options."""

    output_file: Optional[Union[str, bytes, os.PathLike]] = None
    file_suffix: Optional[str] = None
    standalone: bool = True
    template: Optional[str] = None
    binary: bool = False

    @property
    def options(self):
        # Initialise option list
        _options = []

        # Output format
        _options.extend(["-t", self.pandoc_name])

        # Document template
        if self.template:
            _options.extend(["--template", self.template])
        elif self.standalone:
            _options.append("-s")

        # Output channel
        if self.output_file:
            _options.extend(["-o", self.output_file])
        elif self.binary:
            _options.extend(["-o", "-"])

        return _options

    def __str__(self):
        return self.pandoc_name

    def convert(self, input_format, input_text):
        options = ["-f", input_format] + self.options
        decode = not self.binary
        return pandadoc.call_pandoc(
            options=options, input_text=input_text, decode=decode
        )

    def set_output_file(self, path_without_extension):
        self.output_file = f"{path_without_extension}{self.file_suffix}"


@dataclass
class BinaryPandocWriter(PandocWriter):
    binary: bool = True


@dataclass
class TextPandocWriter(PandocWriter):
    binary: bool = False


@dataclass
class PDFPandocWriter(BinaryPandocWriter):
    pandoc_name = "pdf"
    mime_type = "application/pdf"
    file_suffix: Optional[str] = ".pdf"

    @property
    def options(self):
        _options = super().options
        _options.extend(["--pdf-engine", "xelatex"])
        return _options


@dataclass
class MarkdownPandocWriter(TextPandocWriter):
    pandoc_name = "markdown"
    mime_type = "text/markdown"
    file_suffix: Optional[str] = ".md"


@dataclass
class EPUBPandocWriter(BinaryPandocWriter):
    pandoc_name = "epub"
    mime_type = "application/epub+zip"
    file_suffix: Optional[str] = ".epub"


CONFIGURED_WRITERS = [PDFPandocWriter, MarkdownPandocWriter, EPUBPandocWriter]



import datetime

import pytest
from obapi.download import (
    download_essays,
    download_spotify_episodes_json,
    download_youtube_videos_json,
)
from obapi.tidy import (
    _tidy_ob_post_html,
    _tidy_ob_post_object,
    _tidy_spotify_episode_json,
    _tidy_youtube_video_json,
    tidy_essays,
)
from obscraper import get_post_by_url

from markers import require_spotify_api_auth, require_youtube_api_key


class TestTidyYoutubeVideoJSON:
    @require_youtube_api_key
    def test_returns_expected_result_for_example_videos(self):
        # Arrange
        tricks_response = download_youtube_videos_json(["C-gEQdGVXbk"])

        # Act
        tricks_json = _tidy_youtube_video_json(tricks_response["items"][0])

        # Assert
        assert (
            tricks_json["title"] == "10 Python Tips and Tricks For Writing Better Code"
        )
        assert tricks_json["duration"].total_seconds() == pytest.approx(2361)
        assert tricks_json["yt_description"].startswith("This video is")

        # Arrange
        trends_response = download_youtube_videos_json(["e6LOWKVq5sQ"])

        # Act
        trends_json = _tidy_youtube_video_json(trends_response["items"][0])

        # Assert
        assert trends_json["yt_description"] == ""
        assert set(trends_json["classifier_names"]) == {"simpsons", "disco stu"}
        assert trends_json["author_names"] == ["dumbmatter"]


class TestTidySpotifyEpisodeJSON:
    @require_spotify_api_auth
    def test_returns_expected_result_for_example_episode(self):
        # Arrange
        signals_response = download_spotify_episodes_json(["6MAszRR6tdDnMsjgVdw4Jh"])

        # Act
        signals_json = _tidy_spotify_episode_json(signals_response["episodes"][0])

        # Assert
        assert (
            signals_json["title"]
            == "Robin Hanson on Signaling and Self-Deception (Live at Mason Econ)"
        )
        assert signals_json["sp_show_title"] == "Conversations with Tyler"
        assert signals_json["sp_description"].startswith(
            "If intros aren’t about introductions"
        )


class TestTidyOBPostObject:
    def test_returns_expected_result_for_example_post(self):
        # Arrange
        post_raw = get_post_by_url(
            "https://www.overcomingbias.com/2009/03/signaling-in-economics.html"
        )

        # Act
        post_tidy = _tidy_ob_post_object(post_raw)

        # Assert
        assert post_tidy["title"] == "Signaling in Economics"
        assert post_tidy["publish_date"] == datetime.datetime(
            2009, 3, 21, 22, 0, tzinfo=datetime.timezone.utc
        )
        assert post_tidy["ob_post_number"] == 16642
        assert post_tidy["text_plain"].startswith("Arnold Kling cites this")


class TestTidyOBPostHTML:
    @pytest.fixture
    def wrap(self):
        """Wrap a string in HTML tags, to make it look like a post."""

        def _wrap(body):
            return f'<div class="entry-content">{body}</div>'

        return _wrap

    @pytest.mark.parametrize(
        "original,expected",
        [
            ("<p>systems with impressive</p>", "<p>systems with impressive</p>"),
            (
                (
                    "<p>Nonsense string <em>with</em> many nbsps.</p>"
                    "<p> Continues over 2 paragraphs in one string.</p>"
                ),
                (
                    "<p>Nonsense string <em>with</em> many nbsps.</p>"
                    "<p> Continues over 2 paragraphs in one string.</p>"
                ),
            ),
        ],
    )
    def test_replaces_nbsp(self, wrap, original, expected):
        assert _tidy_ob_post_html(wrap(original)) == expected

    @pytest.mark.parametrize(
        "original,expected",
        [
            ("<dl><dd>Example</dd></dl>", "<blockquote><p>Example</p></blockquote>"),
            (
                '<dl style="margin-left: 40px;"><dd>Example</dd></dl>',
                "<blockquote><p>Example</p></blockquote>",
            ),
            (
                "<dl><dd>Example 1</dd><dd>Example 2</dd></dl>",
                "<blockquote><p>Example 1</p><p>Example 2</p></blockquote>",
            ),
            (
                '<p style="padding-left: 30px;">A short string</p>',
                "<blockquote><p>A short string</p></blockquote>",
            ),
            (
                '<div class="blockquote" style="margin-left: 40px;">Values</div>',
                "<blockquote><p>Values</p></blockquote>",
            ),
            (
                '<div class="blockquote">Example</div>',
                "<blockquote><p>Example</p></blockquote>",
            ),
            (
                '<div style="margin-left: 40px;">Example</div>',
                "<blockquote><p>Example</p></blockquote>",
            ),
            (
                '<p style="margin-left: 40px;">Example</p>',
                "<blockquote><p>Example</p></blockquote>",
            ),
            (
                '<p style="padding-left: 30px; text-align: center;">T<sub>ij</sub></p>',
                "<blockquote><p>T<sub>ij</sub></p></blockquote>",
            ),
            (
                '<p style="padding-left: 40px;">Example</p>',
                "<blockquote><p>Example</p></blockquote>",
            ),
            (
                '<p style="padding-left: 37px;">Example</p>',
                '<p style="padding-left: 37px;">Example</p>',
            ),
        ],
    )
    def test_converts_single_blockquotes(self, wrap, original, expected):
        assert _tidy_ob_post_html(wrap(original)) == expected

    @pytest.mark.parametrize(
        "original,expected",
        [
            (
                '<p style="padding-left: 60px;">Example</p>',
                "<blockquote><blockquote><p>Example</p></blockquote></blockquote>",
            )
        ],
    )
    def test_converts_double_blockquotes(self, wrap, original, expected):
        assert _tidy_ob_post_html(wrap(original)) == expected

    def test_unwraps_apple_space(self, wrap):
        original = '<p>Example.<span class="Apple-converted-space"> </span></p>'
        expected = "<p>Example. </p>"
        assert _tidy_ob_post_html(wrap(original)) == expected

    def test_removes_msonormal_attributes(self, wrap):
        original = '<p class="MsoNormal">Example</p>'
        expected = "<p>Example</p>"
        assert _tidy_ob_post_html(wrap(original)) == expected

    def test_removes_nobr_tags(self, wrap):
        original = "<p>Example<nobr></nobr><strong><nobr></nobr> </strong></p>"
        expected = "<p>Example<strong> </strong></p>"
        assert _tidy_ob_post_html(wrap(original)) == expected


class TestTidyEssay:
    def test_returns_expected_result_for_example_essay(self):
        # Arrange
        essay_id = "Varytax"
        raw_essays = download_essays([essay_id])

        # Act
        tidied_essays = tidy_essays([essay_id], raw_essays)

        essay = tidied_essays[0]

        # Assert
        assert essay["item_id"] == "Varytax"
        assert essay["title"] == "Diet Pork"



# Generated by Django 4.0.3 on 2022-06-28 14:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0002_alter_contentitem_download_timestamp_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contentitem",
            name="title",
            field=models.CharField(help_text="Title of content.", max_length=200),
        ),
        migrations.AlterField(
            model_name="obcontentitem",
            name="disqus_id",
            field=models.CharField(
                blank=True,
                help_text="Post Disqus API string identifier.",
                max_length=400,
            ),
        ),
        migrations.AlterField(
            model_name="obcontentitem",
            name="item_id",
            field=models.CharField(
                help_text='Post string identifier. E.g. "2006/11/introduction"',
                max_length=300,
                unique=True,
                verbose_name="string ID",
            ),
        ),
    ]



import re

YOUTUBE_VIDEO_ID_REGEX = r"[0-9A-Za-z_-]{10}[048AEIMQUYcgkosw]"
SPOTIFY_EPISODE_ID_REGEX = r"[a-zA-Z0-9]{22}"
OB_POST_NAME_REGEX = r"\d{4}/\d{2}/[a-z0-9-_%]+"
OB_POST_NUMBER_REGEX = r"[0-9]{5}"
ESSAY_ID_REGEX = r"[a-zA-Z0-9-_%]+"


class URLConverter:
    """Base class for converting between URLs and content IDs."""

    def to_id(self, value, group=1):
        # Extract first matching group of regex pattern by default
        match = re.search(self.regex, value)
        if match is None:
            raise ValueError("No match found.")
        item_id = match.group(group)

        return item_id


class YoutubeVideoURLConverter(URLConverter):
    # Youtube video pattern: https://stackoverflow.com/a/37704433
    regex = (
        r"(?:(?:https?:)?//)?(?:(?:www|m)\.)?(?:youtube(?:-nocookie)?\.com|youtu\.be)"
        r"/(?:[\w\-]+\?v=|embed/|v/)?"
        f"({YOUTUBE_VIDEO_ID_REGEX})"
        r"\S*"
    )

    def to_url(self, value):
        return f"https://www.youtube.com/watch?v={value}"


class SpotifyEpisodeURLConverter(URLConverter):
    regex = (
        r"(?:(?:https?:)?//)?(?:open\.spotify\.com/episode/)"
        f"({SPOTIFY_EPISODE_ID_REGEX})"
        r"\S*"
    )

    def to_url(self, value):
        return f"https://open.spotify.com/episode/{value}"


class OBPostLongURLConverter(URLConverter):
    regex = (
        rf"(?:https?://)?(?:www\.)?overcomingbias\.com/({OB_POST_NAME_REGEX})\.html?"
    )

    def to_url(self, value):
        return f"https://www.overcomingbias.com/{value}.html"


class OBPostShortURLConverter(URLConverter):
    regex = (
        r"(?:https?://)?(?:www\.)?overcomingbias\.com/\?p=" f"({OB_POST_NUMBER_REGEX})"
    )

    def to_url(self, value):
        return f"https://www.overcomingbias.com/?p={value}"


class EssayURLConverter(URLConverter):
    regex = (
        r"(?:https?://)"
        r"(?:mason\.gmu\.edu/~rhanson|hanson\.gmu\.edu)"
        rf"/({ESSAY_ID_REGEX})\.html?"
        r"\S*"
    )

    def to_url(self, value):
        return f"https://mason.gmu.edu/~rhanson/{value}.html"



import datetime
import random

import pytest
from obapi.models import OBContentItem


class TestFindByURL:
    pass


@pytest.mark.django_db
def test_update_last_edit_dates(random_obcontentitems):
    items = random_obcontentitems(5)
    true_edit_dates = [item.edit_date for item in items]
    for item in items:
        item.edit_date = datetime.datetime(2010, 1, 1, tzinfo=datetime.timezone.utc)

    item_ids = [item.item_id for item in items]
    items_queryset = OBContentItem.objects.filter(item_id__in=item_ids)

    items_queryset.update_last_edit_dates()

    # Assert
    for i, item in enumerate(items):
        item.refresh_from_db()
        assert item.edit_date == true_edit_dates[i]


@pytest.mark.django_db
def test_download_new_items(obcontent_edit_dates):
    # Arrange - How many posts to run this test on?
    posts_to_create = 5
    # Arrange - select most recent posts
    sorted_edit_dates = dict(
        sorted(obcontent_edit_dates.items(), key=lambda item: item[1], reverse=True)
    )
    edit_date_iterator = iter(sorted_edit_dates.items())
    latest_posts = [next(edit_date_iterator) for i in range(0, posts_to_create)]

    # Act - create a post from a short while back
    oldest_item_id = latest_posts[posts_to_create - 1][0]
    oldest_item_edit_date = latest_posts[posts_to_create - 1][1]
    created_item = OBContentItem.objects.create_item(oldest_item_id)

    # Assert - was it created ok
    assert created_item.item_id == oldest_item_id
    assert created_item.edit_date == oldest_item_edit_date
    assert OBContentItem.objects.count() == 1

    # Act - call download_new_items
    original_download_timestamp = created_item.download_timestamp
    created_count = OBContentItem.objects.download_new_items()

    # Assert - items were created ok
    assert created_count == posts_to_create - 1
    assert OBContentItem.objects.count() == posts_to_create
    assert (
        OBContentItem.objects.get(item_id=oldest_item_id).download_timestamp
        == original_download_timestamp
    )


@pytest.mark.django_db
def test_bulk_create_items(obcontent_edit_dates):
    # Arrange - configure test settings
    batch_size = 2
    posts_to_create = 5

    # Arrange - collect sample post IDs
    all_item_ids = [key for key, value in obcontent_edit_dates.items()]
    sample_ids = random.sample(all_item_ids, posts_to_create)

    # Act - create items
    created_count = OBContentItem.objects.bulk_create_items(
        sample_ids, batch_size=batch_size
    )

    # Assert - items created correctly
    all_items = OBContentItem.objects.all()
    assert all_items.count() == created_count

    if created_count > batch_size:
        assert (
            all_items.latest("download_timestamp").download_timestamp
            > all_items.earliest("download_timestamp").download_timestamp
        )



import datetime
import re

import bleach
import slugify

ISO_8601_DURATION_PATTERN = re.compile(
    (
        r"^P"  # starts with a P
        r"((?P<days>\d+)D)?"  # optional day field must come before T
        r"(T(?=\d+[HMS])((?P<hours>\d+)H)?((?P<minutes>\d+)M)?((?P<seconds>\d+)S)?)?"
        r"$"
    )
)


def parse_duration(duration):
    if (components := ISO_8601_DURATION_PATTERN.search(duration)) is None:
        raise ValueError(f'"{duration}" is not a valid ISO-8601 duration.')
    components = {
        k: int(v if v is not None else 0) for k, v in components.groupdict().items()
    }
    return datetime.timedelta(**components)


def plaintext_to_html(text):
    """Clean and format plaintext, outputting HTML."""
    clean_text = bleach.clean(text)
    html_text = f"<pre>{clean_text}</pre>"
    linkified_text = bleach.linkify(html_text, parse_email=True)
    return linkified_text


def to_slug(text, max_length):
    return slugify.slugify(text, max_length=max_length)


def chunk_iterator(seq, chunk_size):
    """An iterator which yields chunks of size `chunk_size` of a sequence `seq`."""
    for i in range(0, len(seq), chunk_size):
        yield seq[i : i + chunk_size]


def count_words(text, ignore=None):
    """Count the number of words in a string, ignoring some."""
    if ignore is None:
        ignore = []
    text_without_punctuation = re.sub(r"[^a-zA-Z0-9\s]+", "", text)
    text_without_special_chars = re.sub(r"\s+", " ", text_without_punctuation)
    words = len(
        [word for word in str.split(text_without_special_chars) if word not in ignore]
    )
    return words



# Generated by Django 4.0.3 on 2022-06-28 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contentitem",
            name="download_timestamp",
            field=models.DateTimeField(
                editable=False,
                help_text="When the item was last downloaded.",
                null=True,
                verbose_name="download date",
            ),
        ),
        migrations.AlterField(
            model_name="contentitem",
            name="update_timestamp",
            field=models.DateTimeField(
                auto_now=True,
                help_text="When the item was last updated"
                " - either by hand or automatically.",
                verbose_name="update date",
            ),
        ),
        migrations.AlterField(
            model_name="externallink",
            name="url",
            field=models.URLField(help_text="Link URL.", max_length=2048),
        ),
    ]



# Generated by Django 4.0.3 on 2022-08-18 13:55

import django.db.models.deletion
from django.db import migrations, models
from django.db.migrations.operations.models import ModelOptionOperation


def move_data(apps, schema_editor):
    db_alias = schema_editor.connection.alias

    app_name = "obapi"
    text_model_fields = ("text_html", "text_plain", "word_count")
    models_and_fields = {
        "OBContentItem": text_model_fields,
        "EssayContentItem": text_model_fields,
        "SpotifyContentItem": ("listen_count", "duration"),
        "YoutubeContentItem": ("view_count", "duration"),
    }

    for model_name, model_fields in models_and_fields.items():
        ContentModel = apps.get_model(app_name, model_name)
        for item in ContentModel.objects.all():
            for field_name in model_fields:
                setattr(item, field_name, getattr(item, f"{field_name}_tmp"))
            item.save(using=db_alias)


class ChangeModelBase(ModelOptionOperation):
    def __init__(self, name, new_bases=(models.Model,)):
        super().__init__(name)
        self.new_bases = new_bases

    def deconstruct(self):
        kwargs = {"name": self.name, "new_bases": self.new_bases}
        return (self.__class__.__qualname__, [], kwargs)

    def state_forwards(self, app_label, state):
        model_state = state.models[app_label, self.name_lower]
        model_state.bases = self.new_bases
        state.reload_model(app_label, self.name_lower, delay=True)

    def database_forwards(self, app_label, schema_editor, from_state, to_state):
        pass

    def database_backwards(self, app_label, schema_editor, from_state, to_state):
        pass

    def describe(self):
        return "Change bases for model %s" % self.name

    @property
    def migration_name_fragment(self):
        return "remove_%s_bases" % self.name_lower


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0006_alter_sequence_options"),
    ]

    operations = [
        migrations.RenameField(
            model_name="textcontentitem",
            old_name="text_html",
            new_name="text_html_tmp",
        ),
        migrations.RenameField(
            model_name="textcontentitem",
            old_name="text_plain",
            new_name="text_plain_tmp",
        ),
        migrations.RenameField(
            model_name="textcontentitem",
            old_name="word_count",
            new_name="word_count_tmp",
        ),
        migrations.RenameField(
            model_name="audiocontentitem",
            old_name="duration",
            new_name="duration_tmp",
        ),
        migrations.RenameField(
            model_name="audiocontentitem",
            old_name="listen_count",
            new_name="listen_count_tmp",
        ),
        migrations.RenameField(
            model_name="videocontentitem",
            old_name="duration",
            new_name="duration_tmp",
        ),
        migrations.RenameField(
            model_name="videocontentitem",
            old_name="view_count",
            new_name="view_count_tmp",
        ),
        migrations.AddField(
            model_name="essaycontentitem",
            name="text_html",
            field=models.TextField(
                blank=True,
                help_text="Content text HTML.",
                verbose_name="content text HTML",
            ),
        ),
        migrations.AddField(
            model_name="essaycontentitem",
            name="text_plain",
            field=models.TextField(
                blank=True,
                help_text="Content plaintext.",
                verbose_name="content plaintext",
            ),
        ),
        migrations.AddField(
            model_name="essaycontentitem",
            name="word_count",
            field=models.PositiveIntegerField(
                blank=True, help_text="Word count.", null=True
            ),
        ),
        migrations.AddField(
            model_name="obcontentitem",
            name="text_html",
            field=models.TextField(
                blank=True,
                help_text="Content text HTML.",
                verbose_name="content text HTML",
            ),
        ),
        migrations.AddField(
            model_name="obcontentitem",
            name="text_plain",
            field=models.TextField(
                blank=True,
                help_text="Content plaintext.",
                verbose_name="content plaintext",
            ),
        ),
        migrations.AddField(
            model_name="obcontentitem",
            name="word_count",
            field=models.PositiveIntegerField(
                blank=True, help_text="Word count.", null=True
            ),
        ),
        migrations.AddField(
            model_name="spotifycontentitem",
            name="duration",
            field=models.DurationField(
                blank=True, help_text="Duration of episode.", null=True
            ),
        ),
        migrations.AddField(
            model_name="spotifycontentitem",
            name="listen_count",
            field=models.PositiveIntegerField(
                blank=True, help_text="Number of episode listens.", null=True
            ),
        ),
        migrations.AddField(
            model_name="youtubecontentitem",
            name="duration",
            field=models.DurationField(
                blank=True, help_text="Duration of video.", null=True
            ),
        ),
        migrations.AddField(
            model_name="youtubecontentitem",
            name="view_count",
            field=models.PositiveIntegerField(
                blank=True, help_text="Number of video views.", null=True
            ),
        ),
        migrations.RunPython(move_data, migrations.RunPython.noop),
        ChangeModelBase("OBContentItem", (models.Model,)),
        ChangeModelBase("EssayContentItem", (models.Model,)),
        ChangeModelBase("SpotifyContentItem", (models.Model,)),
        ChangeModelBase("YoutubeContentItem", (models.Model,)),
        migrations.RenameField(
            model_name="essaycontentitem",
            old_name="textcontentitem_ptr",
            new_name="contentitem_ptr",
        ),
        migrations.RenameField(
            model_name="obcontentitem",
            old_name="textcontentitem_ptr",
            new_name="contentitem_ptr",
        ),
        migrations.RenameField(
            model_name="spotifycontentitem",
            old_name="audiocontentitem_ptr",
            new_name="contentitem_ptr",
        ),
        migrations.RenameField(
            model_name="youtubecontentitem",
            old_name="videocontentitem_ptr",
            new_name="contentitem_ptr",
        ),
        migrations.AlterField(
            model_name="essaycontentitem",
            name="contentitem_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="obapi.contentitem",
            ),
        ),
        migrations.AlterField(
            model_name="obcontentitem",
            name="contentitem_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="obapi.contentitem",
            ),
        ),
        migrations.AlterField(
            model_name="spotifycontentitem",
            name="contentitem_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="obapi.contentitem",
            ),
        ),
        migrations.AlterField(
            model_name="youtubecontentitem",
            name="contentitem_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                serialize=False,
                to="obapi.contentitem",
            ),
        ),
        ChangeModelBase("OBContentItem", ("obapi.contentitem",)),
        ChangeModelBase("EssayContentItem", ("obapi.contentitem",)),
        ChangeModelBase("SpotifyContentItem", ("obapi.contentitem",)),
        ChangeModelBase("YoutubeContentItem", ("obapi.contentitem",)),
        migrations.DeleteModel(
            name="TextContentItem",
        ),
        migrations.DeleteModel(
            name="AudioContentItem",
        ),
        migrations.DeleteModel(
            name="VideoContentItem",
        ),
    ]



import datetime

import pytest
from obapi.export import EPUBPandocWriter, MarkdownPandocWriter, export_sequence
from obapi.models import ContentItem, Sequence

from markers import require_spotify_api_auth, require_youtube_api_key


@pytest.fixture
def simple_sequence():
    now = datetime.datetime.now(tz=datetime.timezone.utc)
    item_1 = ContentItem.objects.create(
        title="Item 1",
        description_html="<p>Item 1</p><p>Has 2 paragraphs</p>",
        publish_date=now,
    )
    item_2 = ContentItem.objects.create(
        title="Item 2",
        description_html="<p>Item 2</p><p>Has</p><p>3 paragraphs</p>",
        publish_date=now,
    )
    seq = Sequence.objects.create(title="Simple Sequence")
    seq.items.add(item_1, item_2)

    return seq


@pytest.mark.django_db
class TestExport:
    def test_export_simple_sequence_to_markdown(self, simple_sequence, tmp_path):
        # Arrange
        md_writer = MarkdownPandocWriter()
        md_writer.set_output_file(
            path_without_extension=tmp_path / "test_sequence_simple"
        )

        # Act
        export_sequence(simple_sequence, md_writer)

    @require_youtube_api_key
    @require_spotify_api_auth
    def test_export_random_sequence(
        self,
        random_obcontentitems,
        random_youtubecontentitems,
        random_spotifycontentitems,
        random_essaycontentitems,
        tmp_path,
    ):
        # Arrange
        content = [
            *random_youtubecontentitems(5),
            *random_spotifycontentitems(5),
            *random_obcontentitems(5),
            *random_essaycontentitems(5),
        ]

        random_sequence = Sequence.objects.create(title="Random Sequence")
        random_sequence.items.set(content)

        epub_writer = EPUBPandocWriter()
        epub_writer.set_output_file(
            path_without_extension=tmp_path / "test_sequence_random"
        )

        # Act
        export_sequence(random_sequence, epub_writer)

    def test_export_obcontent_sequence(self, random_obcontentitems, tmp_path):
        content = random_obcontentitems(10)

        obcontent_seq = Sequence.objects.create(title="OB Sequence")
        obcontent_seq.items.set(content)

        epub_writer = EPUBPandocWriter()
        epub_writer.set_output_file(
            path_without_extension=tmp_path / "test_sequence_ob"
        )

        # Act
        export_sequence(obcontent_seq, epub_writer)
